from functools import partial
import random
import string
from itertools import cycle
from django.contrib.auth.models import User
from django_dynamic_fixture import G
import pytest
from geo.hierarchy import italy
from geo.models import Currency, Country, AdministrativeAreaType, AdministrativeArea, LocationType, Location
from sample_data_utils.geo import iso2, iso3, isonum, get_codes
from sample_data_utils.text import text
from sample_data_utils.utils import unique, sequence

nextname = partial(sequence, cache={})
countries = iter(get_codes())

# iso2_codes = unique(country)
# iso_number = unique(isonum)


@pytest.fixture
def hierachy():
    G(Country, iso_code='IT', iso_code3='ITA', iso_num=380,
              name='Italy', fullname='Italy, Italian Republic')

    return italy()

@pytest.fixture
def superuser():
    try:
        return User.objects.get(is_superuser=True)
    except User.DoesNotExist:
         return User.objects.create_superuser('superuser','','123')

def subargs(kwargs, prefix):
    prefix = "%s__" % prefix
    ret = {key[len(prefix):]: kwargs.pop(key) for key in kwargs.keys() if key.startswith(prefix)}
    return ret


def currency_factory(**kwargs):
    return G(Currency, code=nextname('CUR').next())


def country_factory(**kwargs):
    names = nextname('Country')

    iso_2 = unique(iso2, 1, cache={})
    iso_3 = unique(iso3, 1, cache={})
    iso_n = unique(isonum, 1, cache={})

    kwargs.setdefault('iso_code', lambda x: iso_2())
    kwargs.setdefault('iso_code3', lambda x: iso_3())
    kwargs.setdefault('num_code', lambda x: '%03d' % iso_n())
    kwargs.setdefault('name', lambda x: next(names))
    kwargs.setdefault('name_en', lambda x: next(names))
    kwargs.setdefault('fullname', lambda x: next(names))
    kwargs.setdefault('currency', None)

    country = G(Country, **kwargs)
    return country


def area_tree_factory(country):
    parent = area_factory(country, name=nextname('RegionArea'), type__name=nextname('Region'))
    area = area_factory(parent=parent, name=nextname('SubRegionArea'), type__name=nextname('SubRegion'))
    location = location_factory(area=area)
    return parent, area, location


def area_type_factory(country, **kwargs):
    kwargs.setdefault('name', nextname('AdministrativeAreaType'))
    kwargs.setdefault('parent', None)
    return AdministrativeAreaType.objects.get_or_create(country=country, **kwargs)[0]


def area_factory(country=None, **kwargs):
    if 'parent' in kwargs:
        country = kwargs['parent'].country
        type_kwargs = subargs(kwargs, 'type')
        type_kwargs['country'] = country
        type_kwargs['parent'] = kwargs['parent'].type
        kwargs['type'] = area_type_factory(**type_kwargs)
    else:
        type_kwargs = subargs(kwargs, 'type')
        type_kwargs['country'] = country
        kwargs['type'] = area_type_factory(**type_kwargs)

    kwargs.setdefault('parent', None)
    kwargs.setdefault('name', nextname('AdministrativeArea'))

    return AdministrativeArea.objects.get_or_create(country=country, **kwargs)[0]


def location_type_factory(**kwargs):
    return G(LocationType, **kwargs)


def location_factory(**kwargs):
    if 'area' in kwargs:
        kwargs['country'] = kwargs['area'].country
    else:
        if not 'country' in kwargs:
            kwargs['country'] = country_factory(**subargs(kwargs, 'country'))
        area_kwargs = subargs(kwargs, 'area')
        area_kwargs['country'] = kwargs['country']
        kwargs['area'] = area_factory(**area_kwargs)
    if 'type' not in kwargs:
        kwargs['type'] = location_type_factory(**subargs(kwargs, 'type'))

    kwargs.setdefault('name', nextname('Location'))
    location = G(Location, **kwargs)
    return location



from modeltranslation.translator import translator, TranslationOptions
from geo.models import Country


class CountryTranslationOptions(TranslationOptions):
    fields = ('name',)


translator.register(Country, CountryTranslationOptions)



# -*- coding: utf-8 -*-
from geo.config import conf
from django.core.urlresolvers import get_callable


def load_currency(stdout=None):
    func = get_callable(conf.LOAD_CURRENCY_FUNC)
    return func(stdout=stdout)


def load_country(stdout=None):
    func = get_callable(conf.LOAD_COUNTRY_FUNC)
    return func(stdout=stdout)



# -*- coding: utf-8 -*-
import pytest
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
from django_dynamic_fixture import G
from geo.models import Country, AdministrativeArea
from .fixtures import hierachy


@pytest.mark.django_db
def test_type_cannot_contains_same_type(hierachy):
    """ a Type cannot contain the same type
    """
    italy, regione, provincia, comune = hierachy

    regione = italy.administrativeareatype_set.get(name='Regione')
    lazio = italy.areas.create(name='Lazio', type=regione)
    with pytest.raises(ValidationError):
        AdministrativeArea.objects.create(name='Lombardia',
                                          parent=lazio,
                                          type=regione)


@pytest.mark.django_db
def test_type_cannot_contains_parent_type(hierachy):
    """ a Type cannot contain the parent type
    """
    italy, regione, provincia, comune = hierachy

    roma = italy.areas.create(name='Comune di Roma', type=comune)
    torino = italy.areas.create(name='Comune di Torino', type=comune)
    roma.parent = torino
    with pytest.raises(ValidationError):
        roma.save()


@pytest.mark.django_db
def test_type_cannot_contains_parent_type(hierachy):
    """ a Type cannot contain the parent type
    """
    italy, regione, provincia, comune = hierachy

    roma = italy.areas.create(name='Comune di Roma', type=comune)
    torino = italy.areas.create(name='Comune di Torino', type=comune)
    roma.parent = torino
    with pytest.raises(ValidationError):
        roma.save()


@pytest.mark.django_db
def test_contains(hierachy):
    italy, regione, provincia, comune = hierachy

    lazio = italy.areas.create(name='Lazio', type=regione)
    rm = italy.areas.create(name='Provincia di Roma', type=provincia, parent=lazio)
    roma = italy.areas.create(name='Comune di Roma', type=comune, parent=rm)

    assert regione in italy
    assert provincia in regione
    assert comune in provincia

    assert roma in rm
    assert rm  in lazio
    assert lazio in italy



# -*- coding: utf-8 -*-
from django import forms
from django.core.exceptions import ValidationError
from django.forms import CharField
from django.forms.widgets import Input, TextInput
from mptt.forms import TreeNodeChoiceField
from geo.models import AdministrativeAreaType, AdministrativeArea, Country, Location


class IsoCode2Field(CharField):

    def __init__(self, max_length=None, min_length=None, *args, **kwargs):
        max_length = 2
        super(IsoCode2Field, self).__init__(max_length, min_length, *args, **kwargs)

    def clean(self, value):
        if value:
            value = value.strip().upper()
        return super(IsoCode2Field, self).clean(value)

class IsoCode3Field(CharField):

    def __init__(self, max_length=None, min_length=None, *args, **kwargs):
        max_length = 3
        super(IsoCode3Field, self).__init__(max_length, min_length, *args, **kwargs)

    def clean(self, value):
        if value:
            value = value.strip().upper()
        return super(IsoCode3Field, self).clean(value)

class IsoNumericField(CharField):

    def __init__(self, max_length=None, min_length=None, *args, **kwargs):
        max_length = 3
        super(IsoNumericField, self).__init__(max_length, min_length, *args, **kwargs)

    def clean(self, value):
        try:
            if value:
                value = '%03d' % int(value.strip().upper())
        except Exception as e:
            raise ValidationError(e)
        return super(IsoNumericField, self).clean(value)

class CountryForm(forms.ModelForm):
    iso_code = IsoCode2Field()
    iso_code3 = IsoCode3Field()
    iso_num = IsoNumericField()

    class Meta:
        model = Country
        widgets = {
            'fullname': TextInput(attrs={'size': 100}),
            # 'iso_code': TextInput(attrs={'size': 2}),
            # 'iso_code3': TextInput(attrs={'size': 3}),
            # 'num_code': TextInput(attrs={'size': 5}),
            # 'tld': TextInput(attrs={'size': 5}),
        }


class AreaForm(forms.ModelForm):
    class Meta:
        model = AdministrativeArea
        widgets = {'name': TextInput(attrs={'size': 100}),
                   'code': TextInput(attrs={'size': 3})}


class AdministrativeAreaForm(forms.ModelForm):
    for_country = None

    def __init__(self, *args, **kwargs):
        super(AdministrativeAreaForm, self).__init__(*args, **kwargs)
        for_country = kwargs.pop('for_country', self.for_country)
        if 'initial' in kwargs:
            self.fields['type'].queryset = AdministrativeAreaType.objects.filter(country=initial.country)
        elif for_country:
            self.fields['type'].queryset = AdministrativeAreaType.objects.filter(country=for_country)

    class Meta:
        model = AdministrativeArea
        widgets = {
            'code': TextInput(attrs={'size': 10}),
        }


class LocationForm(forms.ModelForm):
    area = TreeNodeChoiceField(AdministrativeArea.objects.all(), required=False)

    class Meta:
        model = Location
        widgets = {
            'code': TextInput(attrs={'size': 10}),

        }


def administrativeareaform_factory_for_country(country):
    """ returns a AdministrativeAreaForm type for a specific Country
        ie. only AdministrativeAreaType related to the Counry are allowed.
        used by the admin's inlines

    """
    if country is None:
        return AdministrativeAreaForm
    name = str('%sAdministrativeAreaForm' % country.iso_code)
    args = {'for_country': country}
    return type(name, (AdministrativeAreaForm,), args)



import os
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.management import call_command
from geo.tests.compat import atomic
from django.db.utils import IntegrityError
from django.test import TestCase, TransactionTestCase
from geo.models import Country, AdministrativeArea, Location, Currency


class Test(TestCase):
    #fixtures = ['geo_test.json']


    def setUp(self):
        target = os.path.join(os.path.dirname(__file__), 'geo_test.json')
        call_command('loaddata', target, verbosity=0)
        super(Test, self).setUp()

    def test_validator_country_iso_code(self):
        c = Country(name='a', fullname='aa', iso_code='a', iso3_code='aaa', num_code='123', continent='AS')
        self.assertRaises(ValidationError, c.full_clean, )

    def test_validator_country_iso3_code(self):
        c = Country(name='a', fullname='aa', iso_code='aa', iso3_code='aa', num_code='123', continent='AS')
        self.assertRaises(ValidationError, c.full_clean, )

    def test_validator_country_num_code(self):
        c = Country(name='a', fullname='aa', iso_code='aa', iso3_code='aaa', num_code='aaa', continent='AS')
        self.assertRaises(ValidationError, c.full_clean, )

    def test_consistency1(self):
        """ a Type cannot contain the same type
        """
        italy = Country.objects.get(iso_code='IT')
        regione = italy.administrativeareatype_set.get(name='Regione')
        lazio = italy.areas.get(name='Lazio')
        self.assertRaises(ValidationError, AdministrativeArea.objects.create,
            name='Lombardia',
            parent=lazio,
            type=regione)

    def test_consistency2(self):
        """ a Type cannot contain the parent type
        """
        italy = Country.objects.get(iso_code='IT')
        comune = italy.areas.get(name='Comune di Roma')
        lazio = italy.areas.get(name='Lazio')
        lazio.parent = comune
        self.assertRaises(ValidationError, lazio.save)

    def test_country_inheritance(self):
        """ a Type cannot contain the parent type
        """
        roma_provincia = AdministrativeArea.objects.get(
            country__iso_code='IT', parent__name='Lazio')

        italy = Country.objects.get(iso_code='IT')
        comune = italy.administrativeareatype_set.get(name='Comune')
        new_comune, __ = AdministrativeArea.objects.get_or_create(name='Comune di Viterbo',
            type=comune, parent=roma_provincia)
        self.assertEqual(new_comune.country, roma_provincia.country)

    def test_regione_in_country(self):
        """ a Type cannot contain the parent type
        """
        italy = Country.objects.get(iso_code='IT')
        lazio = italy.areas.get(name='Lazio')
        self.assertTrue(lazio in italy)

    def test_comune_in_regione(self):
        """ a Type cannot contain the parent type
        """
        italy = Country.objects.get(iso_code='IT')
        lazio = italy.areas.get(name='Lazio')
        comune = italy.areas.get(name='Comune di Roma')
        self.assertTrue(comune in lazio)

    def test_unique_together(self):
        italy = Country.objects.get(iso_code='IT')
        regione = italy.administrativeareatype_set.get(name='Regione')
        lazio2 = AdministrativeArea(name='Lazio', country=italy, type=regione)
        self.assertRaises(IntegrityError, lazio2.save)

    def test_same_name_for_location_in_diff_area(self):
        italy = Country.objects.get(iso_code='IT')
        fiumicino = italy.areas.get(name='Comune di Fiumicino')
        # No Roma location within Fiumicino council: just for testing.
        roma2 = Location(name="Roma", country=italy, area=fiumicino)
        roma2.save()
        all_roma = Location.objects.filter(name='Roma')
        self.assertTrue(len(all_roma), 2)
        self.assertTrue(roma2.area, fiumicino)

    def test_unique_together_location_within_same_area(self):
        italy = Country.objects.get(iso_code='IT')
        comune = italy.administrativeareatype_set.get(name='Comune')
        area_comune = AdministrativeArea(name='Area Comune', country=italy, type=comune)
        area_comune.save()
        location_1 = Location(name="Magliana", area=area_comune, country=italy)
        location_1.save()
        # duplicate location in the same area
        location_2 = Location(name="Magliana", area=area_comune, country=italy)
        self.assertRaises(IntegrityError, location_2.save)

    def test_no_area_of_same_name_and_type_within_a_country(self):
        italy = Country.objects.get(iso_code='IT')
        comune = italy.administrativeareatype_set.get(name='Comune')
        an_area = AdministrativeArea(name='Cool Area', country=italy, type=comune)
        an_area.save()
        # duplicate area (name and type) in the same country
        another_area = AdministrativeArea(name='Cool Area', country=italy, type=comune)
        self.assertRaises(IntegrityError, another_area.save)

    def test_natural_key_if_no_lat_lng(self):
        location_1 = Location.objects.get(name="Roma")
        location_2 = Location.objects.get_by_natural_key(*location_1.natural_key())
        self.assertEquals(location_1.pk, location_2.pk)

    def test_natural_key_with_lat_lng(self):
        location_1 = Location.objects.get(name="Bracciano")
        location_2 = Location.objects.get_by_natural_key(*location_1.natural_key())
        self.assertEquals(location_1.pk, location_2.pk)

    def test_administrativearea_natural_key(self):
        area = AdministrativeArea.objects.get(country__iso_code='IT', parent__name='Lazio')
        location_2 = AdministrativeArea.objects.get_by_natural_key(*area.natural_key())
        self.assertEquals(area.pk, location_2.pk)


class TestNaturalKeys(TransactionTestCase):

    def test_dump_and_load(self):
        import os
        from geo.tests.fixtures import location_factory, currency_factory
        from django.db import transaction
        from django.core.management import call_command
        import tempfile

        tempfile = tempfile.mktemp('.json', 'geo')

        location = location_factory()
        currency = currency_factory()

        with open(tempfile, 'w') as fixture_file:
            call_command('dumpdata', use_natural_keys=True, stdout=fixture_file)

        Currency.objects.all().delete()
        Location.objects.all().delete()
        Country.objects.all().delete()

        self.assertEqual(Country.objects.all().count(), 0)
        self.assertEqual(Location.objects.all().count(), 0)
        self.assertEqual(Currency.objects.all().count(), 0)

        try:
            call_command('loaddata', tempfile, use_natural_keys=True, verbosity=0)
        finally:
            os.remove(tempfile)

        self.assertEqual(Country.objects.all().count(), 1)
        self.assertEqual(Location.objects.all().count(), 1)
        self.assertEqual(Currency.objects.all().count(), 2)




# -*- coding: utf-8 -*-

from django.utils.translation import gettext as _
from geo.models import Country


def italy():
    italy = Country.objects.get(iso_code='IT')
    regione, __ = italy.administrativeareatype_set.get_or_create(name='Regione')
    provincia, __ = italy.administrativeareatype_set.get_or_create(name='Provincia',
                                                                   parent=regione)
    comune, __ = italy.administrativeareatype_set.get_or_create(name='Comune',
                                                                parent=provincia)
    return italy, regione, provincia, comune


def usa():
    usa = Country.objects.get(iso_code='US')
    state, __ = italy.administrativeareatype_set.get_or_create(name='State')
    county, __ = italy.administrativeareatype_set.get_or_create(name='County',
                                                                parent=state)
    township, __ = italy.administrativeareatype_set.get_or_create(name='Township',
                                                                  parent=county)
    return usa, state, county, township




# -*- coding: utf-8 -*-
'''
Created on May 7, 2010

@author: sax
'''
import functools
import warnings
from datetime import datetime
from timezone_field import TimeZoneField
from uuidfield import UUIDField
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.validators import MinLengthValidator, RegexValidator
from django.db import models
from django.db.models.manager import Manager
from django.utils.translation import ugettext_lazy as _
import logging
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey

logger = logging.getLogger("geo")


class CurrencyManager(Manager):
    use_for_related_fields = True

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class Currency(models.Model):
    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")
    iso_code = models.CharField(max_length=5, db_index=True, unique=True, help_text="ISO 4217 code")
    numeric_code = models.CharField(max_length=5, unique=True, help_text="ISO 4217 code")
    decimals = models.IntegerField(default=0)
    name = models.CharField(max_length=100)
    symbol = models.CharField(max_length=5, blank=True, null=True)
    objects = CurrencyManager()

    class Meta:
        app_label = 'geo'
        ordering = ['iso_code', ]
        verbose_name_plural = 'Currencies'

    def __unicode__(self):
        return unicode("%s (%s)" % (self.iso_code, self.name))

    def natural_key(self):
        return self.uuid.hex,


CONTINENTS = (
    ('AF', _('Africa')),
    ('AN', _('Antartica')),
    ('AS', _('Asia')),
    ('EU', _('Europe')),
    ('NA', _('North America')),
    ('OC', _('Oceania')),
    ('SA', _('South America')),
)


class CountryManager(Manager):
    use_for_related_fields = True

    def valid(self):
        return self.get_query_set().filter(expired__isnull=True)

    def _by_continent(self, continent):
        return self.valid().filter(continent=continent)

    def asia(self):
        return self._by_continent('AS')

    def europa(self):
        return self._by_continent('EU')

    def north_america(self):
        return self._by_continent('NA')

    def south_america(self):
        return self._by_continent('SA')

    def oceania(self):
        return self._by_continent('OC')

    def antartica(self):
        return self._by_continent('AN')

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class UNRegion(models.Model):
    code = models.CharField(max_length=5, unique=True, blank=False, null=False, db_index=True)

    name = models.CharField(max_length=100)
    last_update = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'geo'
        verbose_name_plural = _('UN Regions')
        ordering = ['name']

    def __unicode__(self):
        return self.name

class Country(models.Model):
    """ Model for the country of origin.
    """
    iso_code = models.CharField(max_length=2, unique=True, blank=False, null=False, db_index=True,
                                help_text='ISO 3166-1 alpha 2', validators=[MinLengthValidator(2)])
    iso_code3 = models.CharField(max_length=3, unique=True, blank=False, null=False, db_index=True,
                                 help_text='ISO 3166-1 alpha 3', validators=[MinLengthValidator(3)])
    iso_num = models.CharField(max_length=3, unique=True, blank=False, null=False,
                               help_text='ISO 3166-1 numeric', validators=[RegexValidator('\d\d\d')])
    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")

    name = models.CharField(max_length=100, db_index=True)

    fullname = models.CharField(max_length=100, db_index=True)

    region = models.ForeignKey(UNRegion, blank=True, null=True, default=None)
    continent = models.CharField(choices=CONTINENTS, max_length=2)
    currency = models.ForeignKey(Currency, blank=True, null=True)

    tld = models.CharField(help_text='Internet tld', max_length=5, blank=True, null=True)
    phone_prefix = models.CharField(help_text='Phone prefix number', max_length=20, blank=True, null=True)

    timezone = TimeZoneField(blank=True, null=True, default=None)
    expired = models.DateField(blank=True, null=True, default=None)

    lat = models.DecimalField("Latitude", max_digits=18, decimal_places=12, blank=True, null=True)
    lng = models.DecimalField("Longitude", max_digits=18, decimal_places=12, blank=True, null=True)

    last_update = models.DateTimeField(auto_now=True, default=datetime.now)

    fullname.alphabetic_filter = True
    objects = CountryManager()


    class Meta:
        app_label = 'geo'
        verbose_name_plural = _('Countries')
        ordering = ['name']

    def __unicode__(self):
        return u"%s (%s)" % (self.fullname, self.iso_code)

    def clean(self):
        super(Country, self).clean()
        self.iso_code = self.iso_code.upper()
        self.iso_code3 = self.iso_code3.upper()

    def natural_key(self):
        return (self.uuid.hex, )

    def __contains__(self, item):
        if hasattr(item, 'country'):
            return item.country.iso_code == self.iso_code

    def sub(self, type):
        return self.areas.filter(type__name=type)


class AdministrativeAreaTypeManager(TreeManager):
    use_for_related_fields = True

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class AdministrativeAreaType(MPTTModel):
    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")
    name = models.CharField(_('Name'), max_length=100, db_index=True)
    country = models.ForeignKey(Country)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')

    objects = AdministrativeAreaTypeManager()

    class Meta:
        verbose_name = _("Administrative Area Type")
        verbose_name_plural = _("Administrative Area Types")
        app_label = 'geo'
        ordering = ['name']
        order_with_respect_to = 'country'
        unique_together = (('country', 'name'),)

    def __unicode__(self):
        return unicode(self.name)

    def __contains__(self, item):
        if isinstance(item, AdministrativeAreaType) and item.is_child_node():
            return item.is_descendant_of(self)

    def natural_key(self):
        return (self.uuid.hex, )

    natural_key.dependencies = ['geo.country']

    def clean(self):
        if self.parent == self:
            raise ValidationError(_('`%s` cannot contains same type') % self.parent)
        super(AdministrativeAreaType, self).clean()

    def get_or_create(self, **kwargs):
        return AdministrativeArea.objects.get_or_create(country=self.country, type=self, **kwargs)


class AdministrativeAreaManager(TreeManager):
    use_for_related_fields = True

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class AdministrativeArea(MPTTModel):
    """ Administrative areas that can contains other AdministrativeArea and/or Location.
    """

    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")
    name = models.CharField(_('Name'), max_length=255, db_index=True)
    code = models.CharField(_('Code'), max_length=10, blank=True, null=True, db_index=True, help_text='ISO 3166-2 code')
    parent = TreeForeignKey('self', null=True, blank=True, default=None, related_name='areas')
    country = models.ForeignKey(Country, related_name='areas')
    type = models.ForeignKey(AdministrativeAreaType)
    objects = AdministrativeAreaManager()

    class Meta:
        verbose_name = _("Administrative Area")
        verbose_name_plural = _("Administrative Areas")
        unique_together = (('name', 'country', 'type'),)
        app_label = 'geo'
        ordering = ['name']
        order_with_respect_to = 'country'

    def __unicode__(self):
        return unicode(self.name)

    def __repr__(self):
        return "<%s: %s>" % (self.type.name, self.name)

    def natural_key(self):
        return (self.uuid.hex, )

    natural_key.dependencies = ['geo.administrativeareatype', 'geo.country']

    def clean(self):
        if self.parent == self:
            raise ValidationError(_('`%s` cannot contains self') % self)
        if self.parent and self.parent.type == self.type:
            raise ValidationError(_('`%s` cannot contains same type') % self.parent.type)
        if (self.pk and self.parent) and self.parent in self:
            raise ValidationError(_('`%s` cannot contains `%s`') % (self, self.parent))
        super(AdministrativeArea, self).clean()

    def save(self, *args, **kwargs):
        if not self.country_id:
            self.country = self.parent.country
        self.clean()
        super(AdministrativeArea, self).save(*args, **kwargs)

    def __contains__(self, item):
        if isinstance(item, AdministrativeArea) and item.is_child_node():
            return item.is_descendant_of(self)
        elif isinstance(item, Location) and item.area:
            return item.area.is_descendant_of(self)


class LocationTypeManager(models.Manager):
    use_for_related_fields = True

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class LocationType(models.Model):
    """Type of the location (city, village, place, locality, neighbourhood, etc.)
    This is not intended to contain anything inside it.
    """
    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")
    description = models.CharField(unique=True, max_length=100)
    objects = LocationTypeManager()

    class Meta:
        verbose_name_plural = _('Location Types')
        verbose_name = _('Location Type')
        app_label = 'geo'

    def natural_key(self):
        return (self.uuid.hex, )


class LocationManager(models.Manager):
    use_for_related_fields = True

    def get_or_none(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except ObjectDoesNotExist:
            return None

    def get_by_natural_key(self, uuid):
        return self.get(uuid=uuid)


class Location(models.Model):
    """Administrative location (city, place everything with a name and Lat/Lng that
    is not intended to contain anything; use Areas for that).
    """
    NONE = 0
    COUNTRY = 10
    EXACT = 20

    ACCURACY = (
        (NONE, _('None')),
        (COUNTRY, _('Country')),
        (EXACT, _('Exact')))

    country = models.ForeignKey(Country, db_index=True)
    area = models.ForeignKey(AdministrativeArea, db_index=True, blank=True, null=True)
    type = models.ForeignKey(LocationType, blank=True, null=True)

    is_capital = models.BooleanField(default=False, help_text="True if is the capital of `country`")
    is_administrative = models.BooleanField(default=False, help_text="True if is administrative for `area`")
    uuid = UUIDField(auto=True, blank=False, version=1, help_text=_('unique id'), default="")
    name = models.CharField(_('Name'), max_length=255, db_index=True)

    description = models.CharField(max_length=100, blank=True, null=True)
    lat = models.DecimalField(max_digits=18, decimal_places=12, blank=True, null=True)
    lng = models.DecimalField(max_digits=18, decimal_places=12, blank=True, null=True)
    acc = models.IntegerField(choices=ACCURACY, default=NONE, blank=True, null=True,
                              help_text="Define the level of accuracy of lat/lng infos")
    objects = LocationManager()

    class Meta:
        verbose_name_plural = _('Locations')
        verbose_name = _('Location')
        app_label = 'geo'
        ordering = ('name', 'country', )
        order_with_respect_to = 'country'
        unique_together = (('area', 'name'), )

    def __unicode__(self):
        return unicode(self.name)

    def natural_key(self):
        return (self.uuid.hex, )

    natural_key.dependencies = ['geo.administrativearea', 'geo.country', 'geo.locationtype']

    def clean(self):
        if self.area and self.area.country != self.country:
            raise ValidationError('Selected area not in selected country')
        super(Location, self).clean()



from django.contrib.auth.models import User
from django_dynamic_fixture import G
import pytest
from django.core.urlresolvers import reverse
import django_webtest
from .fixtures import superuser, hierachy
from geo.models import Country, AdministrativeAreaType, AdministrativeArea
from tests.fixtures import country_factory, area_tree_factory


@pytest.fixture(scope='function')
def app(request):
    wtm = django_webtest.WebTestMixin()
    wtm._patch_settings()
    wtm._setup_auth()
    request.addfinalizer(wtm._unpatch_settings)
    return django_webtest.DjangoTestApp()


@pytest.mark.django_db(transaction=False)
def test_country_update(app, superuser):
    url = reverse('admin:geo_country_changelist')
    country_factory(n=10)

    res = app.get(url, user=superuser.username)

    res = res.click('^Country-1$')
    res = res.form.submit().follow()


@pytest.mark.django_db(transaction=True)
def test_areatype_update(app, hierachy, superuser):
    italy, regione, provincia, comune = hierachy
    url = reverse('admin:geo_administrativeareatype_changelist')

    res = app.get(url, user=superuser.username)
    res = res.click('^Regione$')
    res = res.form.submit().follow()


@pytest.mark.django_db(transaction=True)
def test_area_update(app, hierachy, superuser):
    italy, regione, provincia, comune = hierachy
    G(AdministrativeArea, type=regione, name='Lazio', country=italy)
    url = reverse('admin:geo_administrativearea_changelist')

    res = app.get(url, user=superuser.username)
    res = res.click('^Lazio$')
    res = res.form.submit().follow()



import os
from tempfile import mktemp

GEO_CACHE = '~build/cache'

DEBUG = True
STATIC_URL = '/static/'

SITE_ID = 1
ROOT_URLCONF = 'tests.urls'
SECRET_KEY = 'abc'
STATIC_ROOT = mktemp('static')
MEDIA_ROOT = mktemp('media')

gettext = lambda s: s
LANGUAGES = (
    ('de', gettext('German')),
    ('en', gettext('English')),
)

INSTALLED_APPS = ['django.contrib.auth',
                  'django.contrib.contenttypes',
                  'django.contrib.sessions',
                  'django.contrib.sites',
                  'django.contrib.messages',
                  'django.contrib.staticfiles',
                  'django.contrib.admin',
                  'south',
                  'modeltranslation',
                  'geo']

TEMPLATE_DIRS = ['tests/templates']

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'full': {
            'format': '%(levelname)-8s: %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'verbose': {
            'format': '%(levelname)-8s: %(asctime)s %(name)-25s %(message)s'
        },
        'simple': {
            'format': '%(levelname)-8s %(asctime)s %(name)-25s %(funcName)s %(message)s'
        },
        'debug': {
            'format': '%(levelno)s:%(levelname)-8s %(name)s %(funcName)s:%(lineno)s:: %(message)s'
        }
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'django.utils.log.NullHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'debug'
        }
    },
    'loggers': {
        'concurrency': {
            'handlers': ['null'],
            'propagate': False,
            'level': 'DEBUG'
        }
    }
}

DBNAME = os.environ.get('DBNAME', 'geo')
db = os.environ.get('DBENGINE', None)
if db == 'pg':
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': DBNAME,
            'HOST': '127.0.0.1',
            'PORT': '',
            'USER': 'postgres',
            'PASSWORD': ''}}
elif db == 'mysql':
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': DBNAME,
            'HOST': '127.0.0.1',
            'PORT': '',
            'USER': 'root',
            'PASSWORD': '',
            'CHARSET': 'utf8',
            'COLLATION': 'utf8_general_ci',
            'TEST_CHARSET': 'utf8',
            'TEST_COLLATION': 'utf8_general_ci'}}
else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': '%s.sqlite' % DBNAME,
            'HOST': '',
            'PORT': ''}}



# -*- coding: utf-8 -*-
from StringIO import StringIO
from contextlib import contextmanager
import os
import requests
import codecs
import sys
import unicodecsv as csv
from django.core.exceptions import ValidationError
from django.db import transaction, IntegrityError
from geo.config import conf
from geo.exceptions import LoadingError
from geo.models import Country, Currency, AdministrativeArea, Location


def splitline(line, sep='\t', ignore='#'):
    if line.startswith(ignore):
        return None
    elif not line.strip():
        return None
    return line.split(sep)


@contextmanager
def open_cached(url, destination=None, rewrite=False):
    filename = destination or url.split('/')[-1]
    fielpath = os.path.join(conf.CACHE, filename)

    if rewrite or not os.path.exists(fielpath):
        r = requests.get(url, stream=True)
        if r.status_code == 200:
            with codecs.open(fielpath, 'wb') as f:
                for chunk in r.iter_content():
                    f.write(chunk)
    f = codecs.open(fielpath, 'r')
    yield f
    f.close()


def load_timezone(overwrite=False, stdout=None):
    from pytz import country_timezones
    # for c in country_timezones()
    stdout = stdout or StringIO()

    def _update(instance, key):
        try:
            instance.timezone = country_timezones[key][0]
            instance.save()
        except KeyError:
            pass

    filters1 = filters2 = filters3 = {}
    if overwrite:
        filters1 = {'timezone__isnull': True}
        filters2 = {'country__timezone__isnull': True}

    for country in Country.objects.filter(**filters1):
        _update(country, country.iso_code)
        stdout.write("Updated timezone for `%s` \n" % unicode(country))

    for aa in AdministrativeArea.objects.filter(**filters2):
        _update(aa, aa.country.iso_code)
        stdout.write("Updated timezone for `%s` \n" % unicode(aa))

    for location in Location.objects.filter(**filters3):
        _update(location, location.country.iso_code)
        stdout.write("Updated timezone for `%s` \n" % unicode(location))


def load_fullnames(stdout=None):
    stdout = stdout or StringIO()
    fielpath = os.path.join(conf.DATA, 'fullnames.csv')
    with codecs.open(fielpath, 'r') as h:
        f = csv.reader(h, encoding='utf-8', delimiter=',', doublequote='"')
        for parts in f:
            code, name, fullname = parts
            try:
                c = Country.objects.get(iso_num=code)
                c.name = name
                c.fullname = fullname
                c.save()
            except Country.DoesNotExist:
                print code, name, fullname


def load_currency_symbols(stdout=None):
    fielpath = os.path.join(conf.DATA, 'symbols.csv')

    with codecs.open(fielpath, 'r') as h:
        f = csv.reader(h, encoding='utf-8', delimiter=';', doublequote='"')
        for parts in f:
            code, symbol = parts
            try:
                c = Currency.objects.get(iso_code=code)
                c.symbol = symbol
                c.save()
            except Currency.DoesNotExist:
                print code
            except Exception as e:
                raise e

def load_country(rewrite=False, stdout=None):
    url = "http://download.geonames.org/export/dump/countryInfo.txt"
    stdout = stdout or StringIO()
    with open_cached(url) as f:
        r = csv.reader(f, encoding='utf-8', delimiter='\t', lineterminator='\n')
        for line in r:
            if line[0][0] == '#':
                continue
            if len(line) == 19:
                iso2, iso3, ison, fips, name, \
                capital, area, population, \
                continent, tld, \
                currencycode, currencyname, \
                phone, postal_code_format, \
                postal_code_regex, \
                languages, \
                geonameid, neighbours, \
                equivalentfipscode = line

                fields = {'iso_code3': iso3, 'iso_code': iso2,
                          'phone_prefix': phone, 'name': name,
                          'fullname': name, 'tld': tld,
                          'continent': continent.upper()}
                c, created = Country.objects.get_or_create(iso_num=ison,
                                                      defaults=fields)

                for field_name, value in fields.items():
                    setattr(c, field_name, value)

                try:
                    curr = Currency.objects.get(iso_code=currencycode)
                    c.currency = curr
                except Currency.DoesNotExist:
                    pass
                c.clean()
                c.save()
                stdout.write("%s\n" % unicode(c))
            else:
                raise LoadingError(line)

    # load_fullnames()


def load_currency(rewrite=False, stdout=None):
    """
        ISO 4217 Currency Codes - Dataset - Frictionless Open Data
    """
    url = "http://www.currency-iso.org/dam/downloads/table_a1.xml"
    from lxml import etree

    stdout = stdout or StringIO()
    with open_cached(url) as f:
        root = etree.fromstring(f.read())
        for element in root.iter("CcyNtry"):
            code = element.find('Ccy')
            if code is not None:
                values = {'numeric_code': element.find('CcyNbr').text,
                          'name': element.find('CcyNm').text,
                          'symbol': ''}

                cur, created = Currency.objects.get_or_create(iso_code=code.text,
                                                              defaults=values)
                if not created:
                    for field_name, value in values.items():
                        setattr(cur, field_name, value)

                try:
                    cur.decimals = int(element.find('CcyMnrUnts').text)
                except ValueError:
                    pass
                cur.save()
                stdout.write('%s\n' % unicode(cur))

    # load_currency_symbols()




# -*- coding: utf-8 -*-
import pytest
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
from django_dynamic_fixture import G
from geo.forms import administrativeareaform_factory_for_country
from geo.models import Country, AdministrativeArea, AdministrativeAreaType
from .fixtures import hierachy


# # @pytest.mark.django_db
# def test_administrativeareatypeform_factory):
#     Form = administrativeareatypeform_factory_for_country(Country())
#     instance  = AdministrativeAreaType()
#     form = Form(instance=instance)
#     assert not form.is_valid()

@pytest.mark.django_db
def test_administrativeareaform_factory(hierachy):
    italy, regione, provincia, comune = hierachy
    Form = administrativeareaform_factory_for_country(italy)
    instance = AdministrativeArea()
    form = Form({'name': 'Lazio',
                 'type': regione.pk,
                 'country': italy.pk},
                instance=instance)
    form.full_clean()
    assert form.is_valid()



import pytest
import sys
from geo.utils import load_currency, load_country


@pytest.mark.django_db
def test_load_currency():
    load_currency()


@pytest.mark.django_db
def test_load_country():
    load_country()



from __future__ import absolute_import, unicode_literals
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import get_callable
from django.db.models import Model
from django.utils import six
from django.test.signals import setting_changed
import os


class AppSettings(object):
    """
    Class to manage application related settings
    How to use:

    >>> from django.conf import settings
    >>> settings.APP_OVERRIDE = 'overridden'
    >>> settings.MYAPP_CALLBACK = 100
    >>> class MySettings(AppSettings):
    ...     defaults = {'ENTRY1': 'abc', 'ENTRY2': 123, 'OVERRIDE': None, 'CALLBACK':10}
    ...     def set_CALLBACK(self, value):
    ...         setattr(self, 'CALLBACK', value*2)

    >>> conf = MySettings("APP")
    >>> conf.ENTRY1, settings.APP_ENTRY1
    ('abc', 'abc')
    >>> conf.OVERRIDE, settings.APP_OVERRIDE
    ('overridden', 'overridden')

    >>> conf = MySettings("MYAPP")
    >>> conf.ENTRY2, settings.MYAPP_ENTRY2
    (123, 123)
    >>> conf = MySettings("MYAPP")
    >>> conf.CALLBACK
    200

    """
    defaults = {
        'CACHE': './cache',
        'DATA' : os.path.join(os.path.dirname(__file__), 'data'),
        'LOAD_CURRENCY_FUNC': 'geo.loaders.load_currency',
        'LOAD_COUNTRY_FUNC': 'geo.loaders.load_country',
    }

    def __init__(self, prefix):
        """
        Loads our settings from django.conf.settings, applying defaults for any
        that are omitted.
        """
        self.prefix = prefix
        from django.conf import settings

        for name, default in self.defaults.items():
            prefix_name = (self.prefix + '_' + name).upper()
            value = getattr(settings, prefix_name, default)
            self._set_attr(prefix_name, value)
            setattr(settings, prefix_name, value)
            setting_changed.send(self.__class__, setting=prefix_name, value=value, enter=True)

        setting_changed.connect(self._handler)
        if not os.path.isdir(self.CACHE):
            raise ImproperlyConfigured('%s is not a valid directory. Please change `settings.GEO_CACHE` value or create it', self.CACHE)

    def _set_attr(self, prefix_name, value):
        name = prefix_name[len(self.prefix) + 1:]
        if name == 'CACHE':
            value = os.path.abspath(os.path.expanduser(value))
        elif name == 'DATA':
            value = os.path.abspath(os.path.expanduser(value))

        setattr(self, name, value)

    def _handler(self, sender, setting, value, **kwargs):
        """
            handler for ``setting_changed`` signal.

        @see :ref:`django:setting-changed`_
        """
        if setting.startswith(self.prefix):
            self._set_attr(setting, value)


conf = AppSettings('GEO')



import logging

from datetime import datetime
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.core import cache, exceptions
from django.core.urlresolvers import reverse
from geo.models import Country, Currency
from pytz import timezone
from rest_framework import serializers, pagination

#
# Serializers
#

class BaseSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        exclude = ('last_modified_user', 'security_hash')


class CountrySerializer(BaseSerializer):
    class Meta:
        model = Country


class CountryLookupSerializer(BaseSerializer):
    class Meta:
        model = Country
        fields = ('name', 'id')


class CurrencySerializer(BaseSerializer):
    class Meta:
        model = Currency


#
# Views
#
class AbstractListView(ListAPIView):
    permission_classes = (AllowAny, )
    authentication_classes = (TokenAuthentication, SessionAuthentication, BasicAuthentication)
    #permission_classes = (permissions.IsAuthenticated, IsOwner)
    renderer_classes = (UnicodeJSONRenderer, JSONPRenderer, BrowsableAPIRenderer, YAMLRenderer, XMLRenderer)
    paginate_by = 10
    paginate_by_param = 'page_size'
    max_paginate_by = 100


class CountryListView(AbstractListView):
    search_fields = ['name', 'fullname', ]
    serializer_class = CountrySerializer
    pagination_serializer_class = CustomPaginationSerializer
    filter_backends = (filters.DjangoFilterBackend, filters.SearchFilter)
    filter_fields = ('iso_code', 'iso3_code', 'num_code', 'name')

    def get_queryset(self):
        return Country.objects.all().order_by('name')




# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand, CommandError
from optparse import make_option
from django.db import transaction, IntegrityError
from geo.hierarchy import italy
from geo.loaders import load_timezone
from geo.models import Country, Currency
from geo.utils import load_currency, load_country


class Command(BaseCommand):
    args = ''
    option_list = BaseCommand.option_list + (
        make_option('-a', '--all', action='store_true', dest='all'),
        make_option('--it', action='store_true', dest='it'),
        # make_option('-p', '--capitals', action='store_true', dest='capital'),
        # make_option('-z', '--timezone', action='store_true', dest='tz'),
        # make_option('-m', '--currency', action='store_true', dest='currency'),
        # make_option('-r', '--region', action='store_true', dest='region'),
        # make_option('-i', '--ignore-cache', action='store_true', dest='reset'),
    )

    def handle(self, *args, **options):
        load_all = options.get('all')
        all = options.get('all')

        if options.get('it') or all:
            italy()



from collections import OrderedDict
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from django.core.management.commands.dumpdata import Command as DumpCommand
from django.core import serializers
from django.db import router, DEFAULT_DB_ALIAS
from optparse import make_option
import django


class Command(DumpCommand):
    def handle(self, *app_labels, **options):
        if django.VERSION[1] == 4:
            return self._handle(*app_labels, **options)
        else:
            return super(Command, self).handle(*app_labels, **options)

    def _handle(self, *app_labels, **options):
        from django.db.models import get_app, get_apps, get_model

        format = options.get('format')
        indent = options.get('indent')
        using = options.get('database')
        excludes = options.get('exclude')
        show_traceback = options.get('traceback')
        use_natural_keys = options.get('use_natural_keys')
        use_base_manager = options.get('use_base_manager')

        excluded_apps = set()
        excluded_models = set()
        for exclude in excludes:
            if '.' in exclude:
                app_label, model_name = exclude.split('.', 1)
                model_obj = get_model(app_label, model_name)
                if not model_obj:
                    raise CommandError('Unknown model in excludes: %s' % exclude)
                excluded_models.add(model_obj)
            else:
                try:
                    app_obj = get_app(exclude)
                    excluded_apps.add(app_obj)
                except ImproperlyConfigured:
                    raise CommandError('Unknown app in excludes: %s' % exclude)

        if len(app_labels) == 0:
            app_list = OrderedDict((app, None) for app in get_apps() if app not in excluded_apps)
        else:
            app_list = OrderedDict()
            for label in app_labels:
                try:
                    app_label, model_label = label.split('.')
                    try:
                        app = get_app(app_label)
                    except ImproperlyConfigured:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app in excluded_apps:
                        continue
                    model = get_model(app_label, model_label)
                    if model is None:
                        raise CommandError("Unknown model: %s.%s" % (app_label, model_label))

                    if app in app_list.keys():
                        if app_list[app] and model not in app_list[app]:
                            app_list[app].append(model)
                    else:
                        app_list[app] = [model]
                except ValueError:
                    # This is just an app - no model qualifier
                    app_label = label
                    try:
                        app = get_app(app_label)
                    except ImproperlyConfigured:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app in excluded_apps:
                        continue
                    app_list[app] = None

        # Check that the serialization format exists; this is a shortcut to
        # avoid collating all the objects and _then_ failing.
        if format not in serializers.get_public_serializer_formats():
            raise CommandError("Unknown serialization format: %s" % format)

        try:
            serializers.get_serializer(format)
        except KeyError:
            raise CommandError("Unknown serialization format: %s" % format)

        # Now collate the objects to be serialized.
        objects = []
        for model in sort_dependencies(app_list.items()):
            if model in excluded_models:
                continue
            if not model._meta.proxy and router.allow_syncdb(using, model):
                if use_base_manager:
                    objects.extend(model._base_manager.using(using).all())
                else:
                    objects.extend(model._default_manager.using(using).all())

        try:
            return serializers.serialize(format, objects, indent=indent,
                                         use_natural_keys=use_natural_keys)
        except Exception, e:
            if show_traceback:
                raise
            raise CommandError("Unable to serialize database: %s" % e)


def sort_dependencies(app_list):
    """Sort a list of app,modellist pairs into a single list of models.

    The single list of models is sorted so that any model with a natural key
    is serialized before a normal model, and any model with a natural key
    dependency has it's dependencies serialized first.
    """
    from django.db.models import get_model, get_models
    # Process the list of models, and get the list of dependencies
    model_dependencies = []
    models = set()
    for app, model_list in app_list:
        if model_list is None:
            model_list = get_models(app)

        for model in model_list:
            models.add(model)
            # Add any explicitly defined dependencies
            if hasattr(model, 'natural_key'):
                deps = getattr(model.natural_key, 'dependencies', [])
                if deps:
                    deps = [get_model(*d.split('.')) for d in deps]
            else:
                deps = []

            # Now add a dependency for any FK or M2M relation with
            # a model that defines a natural key
            for field in model._meta.fields:
                if hasattr(field.rel, 'to'):
                    rel_model = field.rel.to
                    if hasattr(rel_model, 'natural_key') and rel_model != model:
                        deps.append(rel_model)
            for field in model._meta.many_to_many:
                m2m_model = field.rel.through
                if hasattr(m2m_model, 'natural_key'):
                    # if the m2m model is not in the model_dependencies list,
                    # add it for processing
                    if not any((m[0] == m2m_model for m in model_dependencies)):
                        model_list.append(m2m_model)
            model_dependencies.append((model, deps))

    model_dependencies.reverse()
    # Now sort the models to ensure that dependencies are met. This
    # is done by repeatedly iterating over the input list of models.
    # If all the dependencies of a given model are in the final list,
    # that model is promoted to the end of the final list. This process
    # continues until the input list is empty, or we do a full iteration
    # over the input models without promoting a model to the final list.
    # If we do a full iteration without a promotion, that means there are
    # circular dependencies in the list.
    model_list = []
    while model_dependencies:
        skipped = []
        changed = False
        while model_dependencies:
            model, deps = model_dependencies.pop()

            # If all of the models in the dependency list are either already
            # on the final model list, or not on the original serialization list,
            # then we've found another model with all it's dependencies satisfied.
            found = True
            for candidate in ((d not in models or d in model_list) for d in deps):
                if not candidate:
                    found = False
            if found:
                model_list.append(model)
                changed = True
            else:
                skipped.append((model, deps))
        if not changed:
            raise CommandError("Can't resolve dependencies for %s in serialized app list." %
                               ', '.join('%s.%s' % (model._meta.app_label, model._meta.object_name)
                                         for model, deps in sorted(skipped, key=lambda obj: obj[0].__name__)))
        model_dependencies = skipped

    return model_list



from django.contrib import admin
from django.conf.urls import patterns, include

try:
    from django.apps import AppConfig
    import django

    django.setup()
except ImportError:
    pass

admin.autodiscover()

urlpatterns = patterns('',
                       (r'', include(admin.site.urls)),
                       )



#!/usr/bin/env python
import os
import codecs
from distutils.config import PyPIRCCommand
from setuptools import setup, find_packages

dirname = 'geo'

app = __import__(dirname)


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts), 'r').read()


tests_require = read('geo/requirements/testing.pip')

setup(
    name=app.NAME,
    version=app.get_version(),
    url='https://github.com/saxix/django-geo',
    description="A Django application which manage administrative geographical data.",
    download_url='https://github.com/saxix/django-geo/tarball/master',
    author='sax',
    author_email='sax@os4d.org',
    license='BSD',
    packages=find_packages('.'),
    include_package_data=True,
    platforms=['any'],
    install_requires=read('geo/requirements/install.pip'),
    command_options={
        'build_sphinx': {
            'version': ('setup.py', app.VERSION),
            'release': ('setup.py', app.VERSION)}
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
    ],
    long_description=open('README.rst').read()
)



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand, CommandError
from optparse import make_option
from django.db import transaction, IntegrityError
from geo.loaders import load_timezone
from geo.models import Country, Currency, UNRegion
from geo.utils import load_currency, load_country


Regions = (('OMD', 'West Africa'),
           ('OMJ', 'Southern Africa'),
           ('OMP', 'Latin America & Caribbean'),
           ('OMB', 'Asia'),
           ('OMC', 'Middle East, North Africa, Eastern Europe and Central Asia'), ('OMN', 'Eastern & Central Africa'))


class Command(BaseCommand):
    args = ''
    option_list = BaseCommand.option_list + (
        make_option('-a', '--all', action='store_true', dest='all'),
        make_option('-c', '--country', action='store_true', dest='country'),
        make_option('-p', '--capitals', action='store_true', dest='capital'),
        make_option('-z', '--timezone', action='store_true', dest='tz'),
        make_option('-m', '--currency', action='store_true', dest='currency'),
        make_option('-r', '--region', action='store_true', dest='region'),
        # make_option('-i', '--ignore-cache', action='store_true', dest='reset'),
    )

    def handle(self, *args, **options):
        load_all = options.get('all')
        # reset = options.get('currency')
        if load_all:
            region = tz = country = capital = currency = region = True
        else:
            country = options.get('country')
            region = options.get('region')
            capital = options.get('capital')
            tz = options.get('tz')
            currency = options.get('currency')

        if currency:
            self.stdout.write("Loading currencies...")
            load_currency(self.stdout)
        if region:
            for code, name in Regions:
                UNRegion.objects.get_or_create(name=name, code=code)
        if country:
            self.stdout.write("Loading countries...")
            load_country(self.stdout)
        if tz:
            self.stdout.write("Loading timezones...")
            load_timezone(self.stdout)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Currency'
        db.create_table(u'geo_currency', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('iso_code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=5, unique=True)),
            ('numeric_code', self.gf('django.db.models.fields.CharField')(max_length=5, unique=True)),
            ('decimals', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('symbol', self.gf('django.db.models.fields.CharField')(blank=True, max_length=5, null=True)),
        ))
        db.send_create_signal('geo', ['Currency'])

        # Adding model 'UNRegion'
        db.create_table(u'geo_unregion', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=5, unique=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('last_update', self.gf('django.db.models.fields.DateTimeField')(blank=True, auto_now=True)),
        ))
        db.send_create_signal('geo', ['UNRegion'])

        # Adding model 'Country'
        db.create_table(u'geo_country', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('iso_code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=2, unique=True)),
            ('iso_code3', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=3, unique=True)),
            ('iso_num', self.gf('django.db.models.fields.CharField')(max_length=3, unique=True)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('name', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=100)),
            ('fullname', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=100)),
            ('region', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, null=True, to=orm['geo.UNRegion'], default=None)),
            ('continent', self.gf('django.db.models.fields.CharField')(max_length=2)),
            ('currency', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, null=True, to=orm['geo.Currency'])),
            ('tld', self.gf('django.db.models.fields.CharField')(blank=True, max_length=5, null=True)),
            ('phone_prefix', self.gf('django.db.models.fields.CharField')(blank=True, max_length=20, null=True)),
            ('timezone', self.gf('timezone_field.fields.TimeZoneField')(blank=True, null=True, default=None)),
            ('expired', self.gf('django.db.models.fields.DateField')(blank=True, null=True, default=None)),
            ('lat', self.gf('django.db.models.fields.DecimalField')(blank=True, decimal_places=12, max_digits=18, null=True)),
            ('lng', self.gf('django.db.models.fields.DecimalField')(blank=True, decimal_places=12, max_digits=18, null=True)),
            ('last_update', self.gf('django.db.models.fields.DateTimeField')(blank=True, auto_now=True, default=datetime.datetime.now)),
        ))
        db.send_create_signal('geo', ['Country'])

        # Adding model 'AdministrativeAreaType'
        db.create_table(u'geo_administrativeareatype', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('name', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=100)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['geo.Country'])),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, null=True, to=orm['geo.AdministrativeAreaType'], related_name='children')),
            ('_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            (u'lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('geo', ['AdministrativeAreaType'])

        # Adding unique constraint on 'AdministrativeAreaType', fields ['country', 'name']
        db.create_unique(u'geo_administrativeareatype', ['country_id', 'name'])

        # Adding model 'AdministrativeArea'
        db.create_table(u'geo_administrativearea', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('name', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=255)),
            ('code', self.gf('django.db.models.fields.CharField')(db_index=True, blank=True, max_length=10, null=True)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, null=True, to=orm['geo.AdministrativeArea'], related_name='areas', default=None)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['geo.Country'], related_name='areas')),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['geo.AdministrativeAreaType'])),
            ('_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            (u'lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('geo', ['AdministrativeArea'])

        # Adding unique constraint on 'AdministrativeArea', fields ['name', 'country', 'type']
        db.create_unique(u'geo_administrativearea', ['name', 'country_id', 'type_id'])

        # Adding model 'LocationType'
        db.create_table(u'geo_locationtype', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=100, unique=True)),
        ))
        db.send_create_signal('geo', ['LocationType'])

        # Adding model 'Location'
        db.create_table(u'geo_location', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['geo.Country'])),
            ('area', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, null=True, to=orm['geo.AdministrativeArea'])),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, null=True, to=orm['geo.LocationType'])),
            ('is_capital', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_administrative', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('uuid', self.gf('uuidfield.fields.UUIDField')(blank=True, max_length=32, unique=True, default='')),
            ('name', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=255)),
            ('description', self.gf('django.db.models.fields.CharField')(blank=True, max_length=100, null=True)),
            ('lat', self.gf('django.db.models.fields.DecimalField')(blank=True, decimal_places=12, max_digits=18, null=True)),
            ('lng', self.gf('django.db.models.fields.DecimalField')(blank=True, decimal_places=12, max_digits=18, null=True)),
            ('acc', self.gf('django.db.models.fields.IntegerField')(blank=True, null=True, default=0)),
            ('_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('geo', ['Location'])

        # Adding unique constraint on 'Location', fields ['area', 'name']
        db.create_unique(u'geo_location', ['area_id', 'name'])


    def backwards(self, orm):
        # Removing unique constraint on 'Location', fields ['area', 'name']
        db.delete_unique(u'geo_location', ['area_id', 'name'])

        # Removing unique constraint on 'AdministrativeArea', fields ['name', 'country', 'type']
        db.delete_unique(u'geo_administrativearea', ['name', 'country_id', 'type_id'])

        # Removing unique constraint on 'AdministrativeAreaType', fields ['country', 'name']
        db.delete_unique(u'geo_administrativeareatype', ['country_id', 'name'])

        # Deleting model 'Currency'
        db.delete_table(u'geo_currency')

        # Deleting model 'UNRegion'
        db.delete_table(u'geo_unregion')

        # Deleting model 'Country'
        db.delete_table(u'geo_country')

        # Deleting model 'AdministrativeAreaType'
        db.delete_table(u'geo_administrativeareatype')

        # Deleting model 'AdministrativeArea'
        db.delete_table(u'geo_administrativearea')

        # Deleting model 'LocationType'
        db.delete_table(u'geo_locationtype')

        # Deleting model 'Location'
        db.delete_table(u'geo_location')


    models = {
        'geo.administrativearea': {
            'Meta': {'object_name': 'AdministrativeArea', 'unique_together': "(('name', 'country', 'type'),)", 'ordering': "(u'_order',)"},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'blank': 'True', 'max_length': '10', 'null': 'True'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['geo.Country']", 'related_name': "'areas'"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '255'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.AdministrativeArea']", 'related_name': "'areas'", 'default': 'None'}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['geo.AdministrativeAreaType']"}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.administrativeareatype': {
            'Meta': {'object_name': 'AdministrativeAreaType', 'unique_together': "(('country', 'name'),)", 'ordering': "(u'_order',)"},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['geo.Country']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.AdministrativeAreaType']", 'related_name': "'children'"}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.country': {
            'Meta': {'object_name': 'Country', 'ordering': "['name']"},
            'continent': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'currency': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.Currency']"}),
            'expired': ('django.db.models.fields.DateField', [], {'blank': 'True', 'null': 'True', 'default': 'None'}),
            'fullname': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '2', 'unique': 'True'}),
            'iso_code3': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '3', 'unique': 'True'}),
            'iso_num': ('django.db.models.fields.CharField', [], {'max_length': '3', 'unique': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'auto_now': 'True', 'default': 'datetime.datetime.now'}),
            'lat': ('django.db.models.fields.DecimalField', [], {'blank': 'True', 'decimal_places': '12', 'max_digits': '18', 'null': 'True'}),
            'lng': ('django.db.models.fields.DecimalField', [], {'blank': 'True', 'decimal_places': '12', 'max_digits': '18', 'null': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100'}),
            'phone_prefix': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '20', 'null': 'True'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.UNRegion']", 'default': 'None'}),
            'timezone': ('timezone_field.fields.TimeZoneField', [], {'blank': 'True', 'null': 'True', 'default': 'None'}),
            'tld': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '5', 'null': 'True'}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.currency': {
            'Meta': {'object_name': 'Currency', 'ordering': "['iso_code']"},
            'decimals': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '5', 'unique': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'numeric_code': ('django.db.models.fields.CharField', [], {'max_length': '5', 'unique': 'True'}),
            'symbol': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '5', 'null': 'True'}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.location': {
            'Meta': {'object_name': 'Location', 'unique_together': "(('area', 'name'),)", 'ordering': "(u'_order',)"},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'acc': ('django.db.models.fields.IntegerField', [], {'blank': 'True', 'null': 'True', 'default': '0'}),
            'area': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.AdministrativeArea']"}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['geo.Country']"}),
            'description': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '100', 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_administrative': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_capital': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'lat': ('django.db.models.fields.DecimalField', [], {'blank': 'True', 'decimal_places': '12', 'max_digits': '18', 'null': 'True'}),
            'lng': ('django.db.models.fields.DecimalField', [], {'blank': 'True', 'decimal_places': '12', 'max_digits': '18', 'null': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '255'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'null': 'True', 'to': "orm['geo.LocationType']"}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.locationtype': {
            'Meta': {'object_name': 'LocationType'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uuid': ('uuidfield.fields.UUIDField', [], {'blank': 'True', 'max_length': '32', 'unique': 'True', 'default': "''"})
        },
        'geo.unregion': {
            'Meta': {'object_name': 'UNRegion', 'ordering': "['name']"},
            'code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '5', 'unique': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'auto_now': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['geo']


from django_dynamic_fixture.fixture_algorithms.sequential_fixture import SequentialDataFixture


class DataFixture(SequentialDataFixture):

    def uuidfield_config(self, field, key):

        if hasattr(field, '_create_uuid'):
            value = field._create_uuid()
        elif hasattr(field, 'create_uuid'):
            value = field.create_uuid()
        else:
            value = None
        return str(value)




def test_naturalkeys():
    pass

# class TestNaturalKeys(TransactionTestCase):
#
#     def test_dump_and_load(self):
#         import os
#         from geo.tests.fixtures import location_factory, currency_factory
#         from django.db import transaction
#         from django.core.management import call_command
#         import tempfile
#
#         tempfile = tempfile.mktemp('.json', 'geo')
#
#         location = location_factory()
#         currency = currency_factory()
#
#         with open(tempfile, 'w') as fixture_file:
#             call_command('dumpdata', use_natural_keys=True, stdout=fixture_file)
#
#         Currency.objects.all().delete()
#         Location.objects.all().delete()
#         Country.objects.all().delete()
#
#         self.assertEqual(Country.objects.all().count(), 0)
#         self.assertEqual(Location.objects.all().count(), 0)
#         self.assertEqual(Currency.objects.all().count(), 0)
#
#         try:
#             call_command('loaddata', tempfile, use_natural_keys=True, verbosity=0)
#         finally:
#             os.remove(tempfile)
#
#         self.assertEqual(Country.objects.all().count(), 1)
#         self.assertEqual(Location.objects.all().count(), 1)
#         self.assertEqual(Currency.objects.all().count(), 2)
#
